<?php

namespace Intergo\RabbitQueue\Queue;

use Illuminate\Contracts\Debug\ExceptionHandler;
use PhpAmqpLib\Channel\AMQPChannel;
use PhpAmqpLib\Exception\AMQPChannelClosedException;
use PhpAmqpLib\Exception\AMQPConnectionClosedException;
use Intergo\RabbitQueue\Queue\RabbitMQQueue as BaseRabbitMQQueue;
use PhpAmqpLib\Exception\AMQPIOException;
use PhpAmqpLib\Exception\AMQPTimeoutException;

class IntergoRabbitMQQueue extends BaseRabbitMQQueue
{
    protected $max_retries = 3;

    protected $retries = 0;

    protected function publishBasic($msg, $exchange = '', $destination = '', $mandatory = false, $immediate = false, $ticket = null): void
    {
        try {
            parent::publishBasic($msg, $exchange, $destination, $mandatory, $immediate, $ticket);
        } catch (AMQPConnectionClosedException|AMQPChannelClosedException|AMQPTimeoutException $exception) {
            app(ExceptionHandler::class)->report($exception);
            $this->countNumberOfRetries($exception);
            $this->reconnect();
            parent::publishBasic($msg, $exchange, $destination, $mandatory, $immediate, $ticket);
        }
    }

    protected function publishBatch($jobs, $data = '', $queue = null): void
    {
        try {
            parent::publishBatch($jobs, $data, $queue);
            $this->retries = 0;
        } catch (AMQPConnectionClosedException|AMQPChannelClosedException|AMQPTimeoutException $exception) {
            app(ExceptionHandler::class)->report($exception);
            $this->countNumberOfRetries($exception);
            $this->reconnect();
            parent::publishBatch($jobs, $data, $queue);
        }
    }

    protected function createChannel(): AMQPChannel
    {
        try {
            $channel = parent::createChannel();
            $this->retries = 0;
            return $channel;
        } catch (AMQPConnectionClosedException|AMQPChannelClosedException|AMQPIOException|AMQPTimeoutException $exception) {
            app(ExceptionHandler::class)->report($exception);
            $this->countNumberOfRetries($exception);
            $this->reconnect();
            return parent::createChannel();
        }
    }

    /**
     * Count how many retries we did and if they exceed the max_retries throw the exception
     * @param $exception
     * @return void
     */
    protected function countNumberOfRetries($exception)
    {
        $this->retries++;
        if($this->retries >= $this->max_retries){
            throw $exception;
        }
    }

}
